@@ -6,10 +6,7 @@
 # SPDX-License-Identifier: BSD-2-Clause-Patent
 ##
 import logging
-try:
-    from StringIO import StringIO
-except ImportError:
-    from io import StringIO
+import io
 
 
 class StringStreamHandler(logging.StreamHandler):
@@ -17,7 +14,7 @@ class StringStreamHandler(logging.StreamHandler):
 
     def __init__(self):
         logging.Handler.__init__(self)
-        self.stream = StringIO()
+        self.stream = io.StringIO()
 
     def handle(self, record):
         """
@@ -44,7 +41,7 @@ class StringStreamHandler(logging.StreamHandler):
         self.stream.seek(0, 0)
 
     def seek_end(self):
-        self.stream.seek(2, 0)
+        self.stream.seek(0, io.SEEK_END)
 
     def seek(self, offset, whence):
         return self.stream.seek(offset, whence)
