@@ -11,10 +11,13 @@ import datetime
 import struct
 import hashlib
 import uuid
+import io
+from typing import BinaryIO
+from operator import attrgetter
 from edk2toollib.uefi.wincert import WinCert, WinCertUefiGuid
 from edk2toollib.utility_functions import PrintByteList
 
-# spell-checker: ignore decodefs, createfs
+# spell-checker: ignore decodefs, createfs, deduplicated, deduplication
 
 '''
 Structures definition based on UEFI specification (UEFI 2.7)
@@ -34,13 +37,20 @@ Sha256Oid = [0x60, 0x86, 0x48, 0x01, 0x65, 0x03, 0x04, 0x02, 0x01]
 
 class EfiSignatureDataEfiCertX509(object):
     STATIC_STRUCT_SIZE = 16
+    FIXED_SIZE = False
 
     #
     # decodefs is a filestream object of binary content that is the structure encoded
     # decodesize is number of bytes to decode as the EFI_SIGNATURE_DATA object (guid + x509 data)
     # createfs is a filestream object that is the DER encoded x509 cert
+    # cert is a bytes object containing the x509 certificate to initialize this object
     # sigowner is the uuid object of the signature owner guid
-    def __init__(self, decodefs=None, decodesize=0, createfs=None, sigowner=None):
+    def __init__(self,
+                 decodefs: BinaryIO = None,
+                 decodesize=0,
+                 createfs: BinaryIO = None,
+                 cert: bytes = None,
+                 sigowner: uuid = None):
         if(decodefs is not None):
             self.PopulateFromFileStream(decodefs, decodesize)
         elif(createfs is not None):
@@ -53,12 +63,20 @@ class EfiSignatureDataEfiCertX509(object):
             self.SignatureDataSize = end - start
             if(self.SignatureDataSize < 0):
                 raise Exception("Create File Stream has invalid size")
-            self.SignatureData = memoryview(createfs.read(self.SignatureDataSize))
+            self.SignatureData = (createfs.read(self.SignatureDataSize))
+        elif(cert is not None):
+            self.SignatureOwner = sigowner
+            self.SignatureDataSize = len(cert)
+            self.SignatureData = cert
 
         else:
             raise Exception("Invalid Parameters - Not Supported")
 
-    def PopulateFromFileStream(self, fs, decodesize):
+    def __lt__(self, other):
+        """Less-than comparison for sorting. Looks at SignatureData only, not SignatureOwner"""
+        return self.SignatureData < other.SignatureData
+
+    def PopulateFromFileStream(self, fs: BinaryIO, decodesize):
         if(fs is None):
             raise Exception("Invalid File Steam")
 
@@ -79,21 +97,32 @@ class EfiSignatureDataEfiCertX509(object):
 
         self.SignatureOwner = uuid.UUID(bytes_le=fs.read(16))
 
-        # read remainling decode size for x509 data
+        # read remaining decode size for x509 data
         self.SignatureDataSize = decodesize - EfiSignatureDataEfiCertX509.STATIC_STRUCT_SIZE
-        self.SignatureData = memoryview(fs.read(self.SignatureDataSize))
+        self.SignatureData = fs.read(self.SignatureDataSize)
 
-    def Print(self):
-        print("EfiSignatureData - EfiSignatureDataEfiCertX509")
-        print("  Signature Owner:      %s" % str(self.SignatureOwner))
-        print("  Signature Data: ")
-        if(self.SignatureData is None):
-            print("    NONE")
+    def Print(self, compact: bool = False):
+        if not compact:
+            print("EfiSignatureData - EfiSignatureDataEfiCertX509")
+            print("  Signature Owner:      %s" % str(self.SignatureOwner))
+            print("  Signature Data: ")
+            if(self.SignatureData is None):
+                print("    NONE")
+            else:
+                sdl = self.SignatureData
+                if(self.SignatureDataSize != len(sdl)):
+                    raise Exception("Invalid Signature Data Size vs Length of data")
+                PrintByteList(sdl)
         else:
-            sdl = self.SignatureData.tolist()
-            if(self.SignatureDataSize != len(sdl)):
-                raise Exception("Invalid Signature Data Size vs Length of data")
-            PrintByteList(sdl)
+            s = "ESD:EFI_CERT_X509,"
+            s += "%s," % str(self.SignatureOwner)
+            if(self.SignatureData is None):
+                s += 'NONE'
+            else:
+                sdl = self.SignatureData
+                for index in range(len(sdl)):
+                    s += '%02X' % sdl[index]
+            print(s)
 
     def Write(self, fs):
         if(fs is None):
@@ -104,6 +133,12 @@ class EfiSignatureDataEfiCertX509(object):
         fs.write(self.SignatureOwner.bytes_le)
         fs.write(self.SignatureData)
 
+    def GetBytes(self) -> bytes:
+        """ Return bytes array produced by Write() """
+        with io.BytesIO() as fs:
+            self.Write(fs)
+            return fs.getvalue()
+
     def GetTotalSize(self):
         return EfiSignatureDataEfiCertX509.STATIC_STRUCT_SIZE + self.SignatureDataSize
 
@@ -113,22 +148,31 @@ class EfiSignatureDataEfiCertX509(object):
 #
 class EfiSignatureDataEfiCertSha256(object):
     STATIC_STRUCT_SIZE = 16 + hashlib.sha256().digest_size  # has guid and array
+    FIXED_SIZE = True
 
     #
     # decodefs is a filestream object of binary content that is the structure encoded
     # createfs is a filestream object of binary that is to be hashed to create the signature data
-    # digest is a byte array that contains the hash value for new signature data
+    # digest is a bytes object that contains the hash value for new signature data
     # sigowner is the uuid object of the signature owner guid
-    def __init__(self, decodefs=None, createfs=None, digest=None, sigowner=None):
+    def __init__(self,
+                 decodefs: BinaryIO = None,
+                 createfs: BinaryIO = None,
+                 digest: bytes = None,
+                 sigowner: uuid = None):
         if(decodefs is not None):
             self.PopulateFromFileStream(decodefs)
         elif(createfs is not None):
             # create a new one
             self.SignatureOwner = sigowner
-            self.SignatureData = memoryview(hashlib.sha256(createfs.read()).digest())
+            self.SignatureData = (hashlib.sha256(createfs.read()).digest())
         elif(digest is not None):
-            self.SignatureOwner = uuid.UUID(sigowner)
-            self.SignatureData = memoryview(digest)
+            digest_length = len(digest)
+            if(digest_length != hashlib.sha256().digest_size):
+                raise Exception("Invalid digest length (found / expected): (%d / %d)",
+                                digest_length, hashlib.sha256().digest_size)
+            self.SignatureOwner = sigowner
+            self.SignatureData = digest
         else:
             raise Exception("Invalid Parameters - Not Supported")
 
@@ -147,19 +191,30 @@ class EfiSignatureDataEfiCertSha256(object):
 
         self.SignatureOwner = uuid.UUID(bytes_le=fs.read(16))
 
-        self.SignatureData = memoryview(fs.read(hashlib.sha256().digest_size))
+        self.SignatureData = fs.read(hashlib.sha256().digest_size)
 
-    def Print(self):
-        print("EfiSignatureData - EfiSignatureDataEfiCertSha256")
-        print("  Signature Owner:      %s" % str(self.SignatureOwner))
-        print("  Signature Data: ", end="")
-        if(self.SignatureData is None):
-            print(" NONE")
+    def Print(self, compact: bool = False):
+        if not compact:
+            print("EfiSignatureData - EfiSignatureDataEfiCertSha256")
+            print("  Signature Owner:      %s" % str(self.SignatureOwner))
+            print("  Signature Data: ", end="")
+            if(self.SignatureData is None):
+                print(" NONE")
+            else:
+                sdl = self.SignatureData
+                for index in range(len(sdl)):
+                    print("%02X" % sdl[index], end='')
+                print("")
         else:
-            sdl = self.SignatureData.tolist()
-            for index in range(len(sdl)):
-                print("%02X" % sdl[index], end='')
-            print("")
+            s = 'ESD:EFI_CERT_SHA256,'
+            s += "%s," % str(self.SignatureOwner)
+            if(self.SignatureData is None):
+                s += 'NONE'
+            else:
+                sdl = self.SignatureData
+                for index in range(len(sdl)):
+                    s += '%02X' % sdl[index]
+            print(s)
 
     def Write(self, fs):
         if(fs is None):
@@ -170,6 +225,12 @@ class EfiSignatureDataEfiCertSha256(object):
         fs.write(self.SignatureOwner.bytes_le)
         fs.write(self.SignatureData)
 
+    def GetBytes(self) -> bytes:
+        """ Return bytes array produced by Write() """
+        with io.BytesIO() as fs:
+            self.Write(fs)
+            return fs.getvalue()
+
     def GetTotalSize(self):
         return EfiSignatureDataEfiCertSha256.STATIC_STRUCT_SIZE
 
@@ -199,7 +260,7 @@ class EfiSignatureDataFactory(object):
     # from the filestream of an existing auth payload
     #
     @staticmethod
-    def Factory(fs, type, size):
+    def Factory(fs: BinaryIO, type, size):
         if(fs is None):
             raise Exception("Invalid File stream")
 
@@ -241,7 +302,7 @@ class EfiSignatureDataFactory(object):
 class EfiSignatureList(object):
     STATIC_STRUCT_SIZE = 16 + 4 + 4 + 4
 
-    def __init__(self, filestream=None, typeguid=None):
+    def __init__(self, filestream: BinaryIO = None, typeguid: uuid = None):
         if(filestream is None):
 
             # Type of the signature. GUID signature types are defined in below.
@@ -265,7 +326,7 @@ class EfiSignatureList(object):
         else:
             self.PopulateFromFileStream(filestream)
 
-    def PopulateFromFileStream(self, fs):
+    def PopulateFromFileStream(self, fs: BinaryIO):
         if(fs is None):
             raise Exception("Invalid File Steam")
 
@@ -310,27 +371,38 @@ class EfiSignatureList(object):
             a = EfiSignatureDataFactory.Factory(fs, self.SignatureType, self.SignatureSize)
             self.SignatureData_List.append(a)
 
-    def Print(self):
-        print("EfiSignatureList")
-        print("  Signature Type:        %s" % str(self.SignatureType))
-        print("  Signature List Size:   0x%x" % self.SignatureListSize)
-        print("  Signature Header Size: 0x%x" % self.SignatureHeaderSize)
-        print("  Signature Size:        0x%x" % self.SignatureSize)
-        if(self.SignatureHeader is not None):
-            self.SignatureHeader.Print()
+    def Print(self, compact: bool = False):
+        if not compact:
+            print("EfiSignatureList")
+            print("  Signature Type:        %s" % str(self.SignatureType))
+            print("  Signature List Size:   0x%x" % self.SignatureListSize)
+            print("  Signature Header Size: 0x%x" % self.SignatureHeaderSize)
+            print("  Signature Size:        0x%x" % self.SignatureSize)
+            if(self.SignatureHeader is not None):
+                self.SignatureHeader.Print(compact=compact)
+            else:
+                print("  Signature Header:      NONE")
         else:
-            print("  Signature Header:      NONE")
+            csv = "ESL:"
+            csv += "%s" % str(self.SignatureType)
+            csv += ",0x%x" % self.SignatureListSize
+            csv += ",0x%x" % self.SignatureHeaderSize
+            csv += ",0x%x" % self.SignatureSize
+            if(self.SignatureHeader is not None):
+                csv += self.SignatureHeader.Print(compact=compact)
+            else:
+                csv += ",NONE"
+            print(csv)
 
-        for a in self.SignatureData_List:
-            a.Print()
+        if(self.SignatureData_List is not None):
+            for a in self.SignatureData_List:
+                a.Print(compact=compact)
 
     def Write(self, fs):
         if(fs is None):
             raise Exception("Invalid File Output Stream")
         if((self.SignatureHeader is None) and (self.SignatureHeaderSize == -1)):
             raise Exception("Invalid object.  Uninitialized Sig Header")
-        if(self.SignatureData_List is None):
-            raise Exception("Invalid object.  No Sig Data")
 
         fs.write(self.SignatureType.bytes_le)
         fs.write(struct.pack("<I", self.SignatureListSize))
@@ -339,8 +411,15 @@ class EfiSignatureList(object):
         if(self.SignatureHeader is not None):
             self.SignatureHeader.Write(fs)
 
-        for a in self.SignatureData_List:
-            a.Write(fs)
+        if(self.SignatureData_List is not None):
+            for a in self.SignatureData_List:
+                a.Write(fs)
+
+    def GetBytes(self) -> bytes:
+        """ Return bytes array produced by Write() """
+        with io.BytesIO() as fs:
+            self.Write(fs)
+            return fs.getvalue()
 
     def AddSignatureHeader(self, SigHeader, SigSize=0):
         if(self.SignatureHeader is not None):
@@ -355,9 +434,6 @@ class EfiSignatureList(object):
         if(self.SignatureData_List is not None):
             raise Exception("Signature Data List is already initialized")
 
-        if(SigHeader is None) and (SigSize == 0):
-            raise Exception("Invalid parameters.  Can't have no header and 0 Signature Size")
-
         self.SignatureHeader = SigHeader
         if(SigHeader is None):
             self.SignatureHeaderSize = 0
@@ -380,6 +456,196 @@ class EfiSignatureList(object):
         self.SignatureData_List.append(SigDataObject)
         self.SignatureListSize += self.SignatureSize
 
+    def MergeSignatureList(self, esl):
+        """
+        Add the EfiSignatureData entries within the supplied EfiSignatureList to the current object
+
+        No sorting or deduplication is performed, the EfiSignatureData elements are simply appended
+        """
+
+        if not isinstance(esl, EfiSignatureList):
+            raise Exception("Parameter 1 'esl' must be of type EfiSignatureList")
+
+        if(self.SignatureType != esl.SignatureType):
+            raise Exception("Signature Types must match")
+
+        if(self.SignatureHeaderSize > 0
+           or esl.SignatureHeaderSize > 0):
+            raise Exception("Merge does not support Signature Headers")
+        self.SignatureHeaderSize = 0
+
+        if(esl.SignatureListSize == EfiSignatureList.STATIC_STRUCT_SIZE):
+            # supplied EfiSignatureList is empty, return
+            return
+
+        FixedSizeData = esl.SignatureData_List[0].FIXED_SIZE
+        if not FixedSizeData:
+            raise Exception("Can only merge EfiSignatureLists with fixed-size data elements")
+
+        if(self.SignatureData_List is None):
+            self.SignatureData_List = []
+            self.SignatureSize = esl.SignatureSize
+
+        self.SignatureData_List += esl.SignatureData_List
+        self.SignatureListSize += esl.SignatureListSize - EfiSignatureList.STATIC_STRUCT_SIZE
+
+    def SortBySignatureDataValue(self, deduplicate: bool = True):
+        """
+        Sort self's SignatureData_List by SignatureData values (ignores SigOwner) & optionally deduplicate
+
+        When deduplicate is true, remove duplicate SignatureData values from self and return them in an
+        EfiSignatureList.  This EfiSignatureList of duplicates is itself not deduplicated.
+
+        When deduplicate is false, returns an empty EfiSignatureList (has 0 Data elements)
+        """
+
+        # initialize the duplicate list, an EFI_SIGNATURE_LIST with no signature data entries
+        dupes = EfiSignatureList(typeguid=self.SignatureType)
+        dupes.SignatureListSize = EfiSignatureList.STATIC_STRUCT_SIZE
+        dupes.SignatureHeaderSize = self.SignatureHeaderSize
+        dupes.SignatureHeader = self.SignatureHeader
+        dupes.SignatureSize = 0
+        dupes.SignatureData_List = []
+
+        # if nothing to sort, return the empty dupe list
+        if(self.SignatureData_List is None
+           or len(self.SignatureData_List) == 1):
+            return dupes
+
+        self.SignatureData_List.sort(key=attrgetter('SignatureData'))
+
+        if(deduplicate is False):
+            return dupes  # return empty dupe list without performing deduplicate
+
+        # perform deduplicate on self
+        last = len(self.SignatureData_List) - 1  # index of the last item
+        for i in range(last - 1, -1, -1):
+            if self.SignatureData_List[last].SignatureData == self.SignatureData_List[i].SignatureData:
+                dupes.SignatureData_List.insert(0, self.SignatureData_List[last])
+                dupes.SignatureListSize += self.SignatureSize
+                del self.SignatureData_List[last]
+                self.SignatureListSize -= self.SignatureSize
+                last = i
+            else:
+                last = i
+
+        # only initialize dupes.SignatureSize if duplicate elements are present
+        if(len(dupes.SignatureData_List) > 0):
+            dupes.SignatureSize = self.SignatureSize
+
+        return dupes
+
+
+class EfiSignatureDatabase(object):
+    """
+    Concatenation of EFI_SIGNATURE_LISTs, as is returned from UEFI GetVariable() on PK, KEK, db, & dbx
+
+    Useful for parsing and building the contents of the Secure Boot variables
+    """
+
+    def __init__(self, filestream: BinaryIO = None, EslList: [] = None):
+        if filestream:
+            self.EslList = []
+            self.PopulateFromFileStream(filestream)
+        else:
+            self.EslList = [] if EslList is None else EslList
+
+    def PopulateFromFileStream(self, fs: BinaryIO):
+        if(fs is None):
+            raise Exception("Invalid File Stream")
+        self.EslList = []
+        begin = fs.tell()
+        fs.seek(0, io.SEEK_END)
+        end = fs.tell()  # end is offset after last byte
+        fs.seek(begin)
+        while (fs.tell() != end):
+            Esl = EfiSignatureList(fs)
+            self.EslList.append(Esl)
+
+    def Print(self, compact: bool = False):
+        for Esl in self.EslList:
+            Esl.Print(compact=compact)
+
+    def Write(self, fs):
+        if(fs is None):
+            raise Exception("Invalid File Output Stream")
+        for Esl in self.EslList:
+            Esl.Write(fs)
+
+    def GetBytes(self) -> bytes:
+        """ Return bytes array produced by Write() """
+        with io.BytesIO() as fs:
+            self.Write(fs)
+            return fs.getvalue()
+
+    def GetCanonicalAndDupes(self):
+        """
+        Compute and return a tuple containing both a canonicalized database & a database of duplicates
+
+        Returns -> (canonical: EfiSignatureDatabase, duplicates: EfiSignatureDatabase)
+        canonical is an EfiSignatureDatabase where EfiSignatureLists are merged (where possible),
+            deduplicated, & sorted, and the EfiSignatureData elements are also deduplicated & sorted
+        duplicates is an EfiSignatureDatabase with EfiSignatureLists containing any duplicated
+            EfiSignatureData entries (only the data contents are checked for effective equality,
+            signature owner is ignored)
+        """
+
+        # First group EFI_SIGNATURE_LISTS by type, merging them where possible
+
+        sha256esl = None
+        x509eslList = None
+
+        for Esl in self.EslList:
+            if(Esl.SignatureData_List is None):  # discard empty EfiSignatureLists
+                continue
+            if(Esl.SignatureType == EfiSignatureDataFactory.EFI_CERT_SHA256_GUID):
+                if(sha256esl is None):
+                    sha256esl = Esl  # initialize it
+                else:
+                    sha256esl.MergeSignatureList(Esl)
+            elif(Esl.SignatureType == EfiSignatureDataFactory.EFI_CERT_X509_GUID):
+                if(x509eslList is None):
+                    x509eslList = [Esl]  # initialize it
+                else:
+                    x509eslList.append(Esl)
+            else:
+                raise Exception("Unsupported signature type %s", Esl.SignatureType)
+
+        # for each type, sort and de-duplicate, and then populate the respective databases
+        # note the ordering of this section is the prescribed canonical order
+        # first 1 EfiSignatureList for SHA256 hashes, EfiSignatureData elements sorted ascending
+        # followed by EfiSignatureLists for each x509 certificate, sorted ascending by data content
+
+        canonicalDb = EfiSignatureDatabase()
+        duplicatesDb = EfiSignatureDatabase()
+
+        if(sha256esl is not None):
+            dupes = sha256esl.SortBySignatureDataValue(deduplicate=True)
+            canonicalDb.EslList.append(sha256esl)
+            duplicatesDb.EslList.append(dupes)
+
+        if(x509eslList is not None):
+            x509eslList.sort(key=attrgetter('SignatureData_List'))
+            for esl in x509eslList:
+                if not canonicalDb.EslList:
+                    canonicalDb.EslList.append(esl)
+                elif esl == canonicalDb.EslList[-1]:
+                    duplicatesDb.EslList.append(esl)
+                else:
+                    canonicalDb.EslList.append(esl)
+
+        return (canonicalDb, duplicatesDb)
+
+    def GetCanonical(self):
+        """Return a canonicalized EfiSignatureDatabase, see GetCanonicalAndDupes() for more details"""
+        (canonical, dupes) = self.GetCanonicalAndDupes()
+        return canonical
+
+    def GetDuplicates(self):
+        """Return an EfiSignatureDatabase of duplicates, see GetCanonicalAndDupes() for more details"""
+        (canonical, dupes) = self.GetCanonicalAndDupes()
+        return dupes
+
 
 class EfiTime(object):
     STATIC_STRUCT_SIZE = 16
